// CODE GENERATOR

	reg	[5:0]	opCode			;
	reg	[4:0]	destAddr		;
	reg	[4:0]	leftAddr		;
	reg	[4:0]	rightAddr		;
	reg	[10:0]	value			;
	reg [5:0] 	addrCounter 	;
	reg [5:0] 	labelTab[0:63]	;
	
	`include "toyMachineArchitecture.v"
	
	task endLine;
		begin
			dut.progMem[addrCounter] = {opCode		,	
										destAddr	,
										leftAddr	,
										rightAddr	,
										value		}	;
			addrCounter	= addrCounter + 1				;
		end
	endtask

	// sets labelTab in the first pass associating 'counter' with 'labelIndex'
	task LB	; 
		input [4:0] labelIndex; 
		
		labelTab[labelIndex] = addrCounter; 
	endtask
	// uses the content of labelTab in the second pass
	task ULB; 
		input [4:0] labelIndex; 
		
		{rightAddr, value} = labelTab[labelIndex] - addrCounter;	
	endtask

	task NOP; 
		begin 	opCode				= nop	;
				destAddr			= 5'b0	;
				leftAddr			= 5'b0	;
				{rightAddr, value}	= 16'b0	;
				endLine						;
		end
	endtask
	
	task JMP; 
		input	[4:0]	left;
		begin 	opCode				= jmp	;
				destAddr			= 5'b0	;
				leftAddr			= left	;
				{rightAddr, value}	= 16'b0	;
				endLine						;
		end
	endtask
	
	task ZJMP; 
		input	[4:0]	left;
		input	[5:0]	label;
		begin 	opCode		= zjmp	;
				destAddr	= 5'b0	;
				leftAddr	= left	;
				ULB(label)			;
				endLine				;
		end
	endtask
	
	task NZJMP; 
		input	[4:0]	left;
		input	[5:0]	label;
		begin 	opCode		= nzjmp	;
				destAddr	= 5'b0	;
				leftAddr	= left	;
				ULB(label)			;
				endLine				;
		end
	endtask
	
	task RJMP; 
		input	[5:0]	label;
		begin 	opCode		= rjmp	;
				destAddr	= 5'b0	;
				leftAddr	= 5'b0	;
				ULB(label)			;
				endLine				;
		end
	endtask
	
	task EI; 
		begin 	opCode				= ei	;
				destAddr			= 5'b0	;
				leftAddr			= 5'B0	;
				{rightAddr, value}	= 16'b0	;
				endLine						;
		end
	endtask
	
	task DI; 
		begin 	opCode				= di	;
				destAddr			= 5'b0	;
				leftAddr			= 5'B0	;
				{rightAddr, value}	= 16'b0	;
				endLine						;
		end
	endtask
	
	task HALT; 
		begin 	opCode				= halt	;
				destAddr			= 5'b0	;
				leftAddr			= 5'B0	;
				{rightAddr, value}	= 16'b0	;
				endLine						;
		end
	endtask
	
	task NEG; 
		input	[4:0]	dest	;
		input	[4:0]	left	;
		begin 	opCode				= neg	;
				destAddr			= dest	;
				leftAddr			= left	;
				{rightAddr, value}	= 16'b0	;
				endLine						;
		end
	endtask
	
	task AND; 
		input	[4:0]	dest	;
		input	[4:0]	left	;
		input	[4:0]	right	;
		begin 	opCode				= bwand			;
				destAddr			= dest			;
				leftAddr			= left			;
				{rightAddr, value}	= {right, 11'b0};
				endLine								;
		end
	endtask
	
	task OR; 
		input	[4:0]	dest	;
		input	[4:0]	left	;
		input	[4:0]	right	;
		begin 	opCode				= bwor			;
				destAddr			= dest			;
				leftAddr			= left			;
				{rightAddr, value}	= {right, 11'b0};
				endLine								;
		end
	endtask
	
	task XOR; 
		input	[4:0]	dest	;
		input	[4:0]	left	;
		input	[4:0]	right	;
		begin 	opCode				= bwxor			;
				destAddr			= dest			;
				leftAddr			= left			;
				{rightAddr, value}	= {right, 11'b0};
				endLine								;
		end
	endtask
	
	task ADD; 
		input	[4:0]	dest	;
		input	[4:0]	left	;
		input	[4:0]	right	;
		begin 	opCode				= add			;
				destAddr			= dest			;
				leftAddr			= left			;
				{rightAddr, value}	= {right, 11'b0};
				endLine								;
		end
	endtask
	
	task SUB; 
		input	[4:0]	dest	;
		input	[4:0]	left	;
		input	[4:0]	right	;
		begin 	opCode				= sub			;
				destAddr			= dest			;
				leftAddr			= left			;
				{rightAddr, value}	= {right, 11'b0};
				endLine								;
		end
	endtask
	
	task ADDC; 
		input	[4:0]	dest	;
		input	[4:0]	left	;
		input	[4:0]	right	;
		begin 	opCode				= addc			;
				destAddr			= dest			;
				leftAddr			= left			;
				{rightAddr, value}	= {right, 11'b0};
				endLine								;
		end
	endtask
	
	task SUBC; 
		input	[4:0]	dest	;
		input	[4:0]	left	;
		input	[4:0]	right	;
		begin 	opCode				= subc			;
				destAddr			= dest			;
				leftAddr			= left			;
				{rightAddr, value}	= {right, 11'b0};
				endLine								;
		end
	endtask
	
	task MOVE; 
		input	[4:0]	dest;
		input	[4:0]	left;
		begin 	opCode				= move	;
				destAddr			= dest	;
				leftAddr			= left	;
				{rightAddr, value}	= 16'b0	;
				endLine						;
		end
	endtask
	
	task ASHR; 
		input	[4:0]	dest;
		input	[4:0]	left;
		begin 	opCode				= ashr	;
				destAddr			= dest	;
				leftAddr			= left	;
				{rightAddr, value}	= 16'b0;
				endLine						;
		end
	endtask
	
	task VAL; 
		input	[4:0]	dest	;
		input	[15:0]	immVal	;
		begin 	opCode				= val	;
				destAddr			= dest	;
				leftAddr			= 5'B0	;
				{rightAddr, value}	= immVal;
				endLine						;
		end
	endtask
	
	task HVAL; 
		input	[4:0]	dest	;
		input	[4:0]	left	;
		input	[15:0]	immVal	;
		begin 	opCode				= hval	;
				destAddr			= dest	;
				leftAddr			= left	;
				{rightAddr, value}	= immVal;
				endLine						;
		end
	endtask
		
	task RECEIVE; 
		begin 	opCode				= receive	;
				destAddr			= 5'b0		;
				leftAddr			= 5'b0		;
				{rightAddr, value}	= 16'b0		;
				endLine							;
		end
	endtask
	
	task ISSUE; 
		begin 	opCode				= issue	;
				destAddr			= 5'b0	;
				leftAddr			= 5'b0	;
				{rightAddr, value}	= 16'b0	;
				endLine						;
		end
	endtask
	
	task GET; 
		input	[4:0]	dest	;
		begin 	opCode				= get	;
				destAddr			= dest	;
				leftAddr			= 5'b0	;
				{rightAddr, value}	= 16'b0	;
				endLine						;
		end
	endtask
	
	task SEND; 
		input	[4:0]	left;
		begin 	opCode				= send	;
				destAddr			= 5'b0	;
				leftAddr			= left	;
				{rightAddr, value}	= 16'b0	;
				endLine						;
		end
	endtask
	
	task DATARD; 
		input	[4:0]	dest;
		begin 	opCode				= datard;
				destAddr			= dest	;
				leftAddr			= 5'b0	;
				{rightAddr, value}	= 16'b0	;
				endLine						;
		end
	endtask
	
	task DATAWR; 
		input	[4:0]	dest;
		input	[4:0]	left;
		begin 	opCode				= datawr;
				destAddr			= dest	;
				leftAddr			= left	;
				{rightAddr, value}	= 16'b0	;
				endLine						;
		end
	endtask
	
	// RUNNING
	initial begin 	addrCounter	= 0;
					`include "theProgram.v" // first pass
					addrCounter	= 0;
					`include "theProgram.v" // second pass
			end
			