module toyMachineSimulator;
	reg  [15:0] inStream   ; // input stream of data
	reg         empty      ; // inStream has no meaning
	wire        read       ; // read from the sender
	wire [15:0] outStream  ; // output stream of data
	reg         full       ; // the receiver is not able to receive 
	wire        write      ; // write the element form outRegister
	reg         interrupt  ; // interrupt input
	wire        inta       ; // interrupt acknowledge
	reg         reset      ; // reset input
	reg         clock      ; // clock input
	
	integer			i		;
	
	initial	begin 				clock = 0		;
					forever	#1 	clock = ~clock	;
			end
	
	`include "toyMachineCodeGenerator.v"
	
	initial	for (i=0; i<32; i=i+1) 
				$display("progMem[%0d] = %b", i, dut.progMem[i]);			
					  
	initial	begin		reset 		= 1	;
						inStream 	= 0	;
						full		= 0	;
						empty		= 0	;
					#3	reset 		= 0	;
					#380 $stop			;
			end
			
	always @(posedge clock) 
		if (reset)	inStream[2:0] <= 0											;
		 else 		inStream[2:0] <= read ? inStream[2:0] + 1 : inStream[2:0]	; 
			
	toySystem dut(	inStream   , 
					empty      , 
					read       , 
					outStream  , 
					full       , 
					write      , 
					interrupt  , 
					inta       , 
					reset      , 
					clock      );

	initial $monitor("time=%0d \t rst=%b pc=%0d \t cr=%b rf[0]=%0d \t rf[1]=%0d \t rf[2]=%0d \t read=%b inReg=%0d \t write=%b outReg=%0d",
					$time,  	
					reset,
					dut.programCounter,
					dut.tM.carry,
					dut.tM.regFile[0],
					dut.tM.regFile[1],
					dut.tM.regFile[2],
					dut.tM.read,
					dut.tM.inRegister,
					dut.tM.write,
					dut.tM.outRegister);							
endmodule

	