module toySystem
	   (input  [15:0] inStream   , // input stream of data
		input         empty      , // inStream has no meaning 
        output        read       , // read from the sender
		output [15:0] outStream  , // output stream of data
		input         full       , // the receiver is not able to receive 
        output        write      , // write the element form outRegister
		input         interrupt  , // interrupt input
		output        inta       , // interrupt acknowledge
		input         reset      , // reset input
		input         clock      );// clock input
			
	reg		[31:0]	progMem[0:1023]	; // read only
	reg		[31:0]	dataMem[0:1023]	;
	
	wire 	[31:0] 	dataAddr   		; // address for data memory
	wire 	[31:0] 	dataOut    		; // data of be stored in data memory
	wire 	       	store      		; // write in data memory
	wire 	[31:0] 	dataIn     		; // data from data memory
	wire 	[31:0] 	programCounter	; // address for program memory
	wire 	[31:0] 	instruction		; // instruction form the program memory

	toyMachine tM(	inStream   		, 
					empty      		, 
					read       		, 
					outStream  		, 
					full       		,  
					write      		, 
					interrupt  		, 
					inta       		, 
					dataAddr   		, 
					dataOut    		, 
					store      		, 
					dataIn     		, 
					programCounter	, 
					instruction		, 
					reset      		, 
					clock      		);
			
	always @(posedge clock)	dataMem[dataAddr[9:0]] <= dataOut	;
	assign dataIn 		= dataMem[dataAddr[9:0]]		;
	assign instruction 	= progMem[programCounter[9:0]]	;
	
endmodule
